#!/usr/bin/perl
#
# webqcat.pl - CGI script to process and format CueCat barcode data 
#
# Copyright (C) 2000 Micah Dowty <micah@homesoftware.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 

use CGI;
$q = new CGI;
print $q->header('text/html');
$dat = $q->param('data');

print <<EOF;
<html><head><title>WebQcat Report</title></head>
<body bgcolor="#FFFFFF" text="#000000">
<table border=1 cellspacing=2 cellpadding=5>
<tr bgcolor="#8080FF">
  <td><i>Type</i></td>
  <td><i>Code</i></td>
  <td><i>Information</i></td>
</tr>
EOF

# Find and convert barcodes
while ($dat =~ s/(\.[^\.]*\.(.{4}\.[^\.]+\.))//) {
    $_ = $1;
    $originalcode = $2;

    # Larry Wall's tiny decoder
    ($serial,$type,$code) = map { 
    tr/a-zA-Z0-9+-/ -_/; 
    $_ = unpack 'u', chr(32 + length()*3/4) . $_; 
    s/\0+$//; 
    $_ ^= "C" x length; 
    } /\.([^.]+)/g; 

    print "<tr><td>$type</td><td>$code</td><td>\n";
    
    # Convert the IB5 codes to normal IBN codes
    if ($type eq "IB5") {
        $type = "IBN";
        $code = substr($code,0,13);
    }
	
    # Extra information depending on the type of barcode
    if ($type eq "IBN") {
         # It's a book
	 print "Book ISBN code<BR>\n";
	 
	 # Look up extra information about it
	 bookinfo($code);
    }

    # Last resort...
    else {
         print "Other product code<br>\n";
         dcnvinfo($originalcode);
    }

    print "</td></tr>\n";
}

print <<EOF;
</table><center>
  Generated by WebQcat 
  (<a href="http://webqcat.sourceforge.net">http://webqcat.sourceforge.net</a>)
</center>
</body></html>
EOF

# Sub to get some more info about a book
sub bookinfo {
   $server = "shop.barnesandnoble.com";
   $port = 80;
   $document = "/BookSearch/isbnInquiry.asp?ean=$_[0]";
   $document =~ s/\s//g;
   
   ($dummy,$dummy,$dummy,$dummy, @addrs) = gethostbyname($server);
   $remote = pack("S n a4 x8", 2, $port, $addrs[0]);
   socket(S, 2, 1, join("", getprotobyname('tcp'))) || die "socket: $!";
   connect(S, $remote) || die "connect: $!";
   select(S); $| = 1;
   select(STDOUT); $| = 1;
   print S "GET $document HTTP/1.0\r\n\r\n";
   $infopage = join '',<S>;
   close S;
   
   print "<table border=0 cellspacing=2 cellpadding=0><tr>\n";
   
   # Picture
   if ($infopage =~ /(images\.barnesandnoble\.com[^\"]+)/) {
      print "<td><img src=\"http://$1\"></td>\n";
   }
   
   # Gather other information
   @info = ();
   
   if ($infopage =~ /<!-- title -->[^>]*>[^>]*>([^>]+)/) {
      push @info, "Title\\<b><i>$1</i></b>";
   }
   
   if ($infopage =~ /<!-- author -->\s*([^\n]+)/) {
      $_ = $1;
      s/<[^>]*>//g;
      s/&nbsp;/<br>/;
      push @info, "Author\\$_";
   } 
   
   # Display other information
   print "<td><table border=0 cellspacing=4 cellpadding=0>\n";
   foreach (@info) {
      s/\\/:<\/td><td>/;
      print "<tr><td>$_</td></tr>\n";
   }
   print "</table></td>\n";
   
   print "</tr></table><a href=\"http://$server:$port$document\">More information</a>\n"
}

sub dcnvinfo {
   $server = "o.dcnv.com";
   $port = 80;
   $_[0] =~ tr/a-zA-Z/A-Za-z/;
   $document = "/CRQ/1..ACTIVATIONCODE.04.x.$_[0]0";
	    
   ($dummy,$dummy,$dummy,$dummy, @addrs) = gethostbyname($server);
   $remote = pack("S n a4 x8", 2, $port, $addrs[0]);
   socket(S, 2, 1, join("", getprotobyname('tcp'))) || die "socket: $!";
   connect(S, $remote) || die "connect: $!";
   select(S); $| = 1;
   select(STDOUT); $| = 1;
   print S "GET $document HTTP/1.0\r\n\r\n";
   $infopage = join '',<S>;
   close S;
 
   if ($infopage =~ /c-me-register/) {
      print "Unknown";
      return;
   }

   if ($infopage =~ /desc=\s*([^\n]+)/) {
      print "$1<BR>\n";
   }

   if ($infopage =~ /url=\s*([^\n]+)/) {
      print "<a href=\"$1\">$1</a>\n";
   }
}				       


### The End ###
